
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : debug.scm
;; DESCRIPTION : debugging tools
;; COPYRIGHT   : (C) 2002  Joris van der Hoeven, David Allouche
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Output
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (tm-display-error . args)
  (display "TeXmacs] " (current-error-port))
  (for-each (lambda (a)
	      (display a (current-error-port)))
	    args)
  (newline (current-error-port)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Safe evaluation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (apply-with lst lam)
  "Syntactic sugar for multiple return values as a list."
  (apply lam lst))

(define (safe-eval-list lst)
  (catch #t (lambda ()
	      (let ((v (eval lst)))
		(if (list? v)
		    (list #f v)
		    (list #t v))))
	 (lambda args (list #t #f))))

(define (safe-eval-bool lst)
  (catch #t (lambda ()
	      (let ((v (eval lst)))
		(if (boolean? v)
		    (list #f v)
		    (list #t v))))
	 (lambda args (list #t #f))))
  
(define (eval-bool-or-false lst)
  (apply-with (safe-eval-bool lst)
	      (lambda (error? bool)
		(and (not error?) bool))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Debugging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (wrap-catch lam)
  (lambda args
    (catch #t
	   (lambda () (apply lam args))
	   (lambda err
	     (tm-display-error "Guile error: " (list err))
	     (apply throw err)))))

(define (wrap-catch-list expr)
  "Similar to wrap-catch for a scheme expression in list form."
  `(catch #t
	  (lambda () ,expr)
	  (lambda err
	    (tm-display-error "Guile error: " (list err))
	    (apply throw err))))

(define trace-level 0)
(define (trace-indent) (make-string (* 4 trace-level) #\space))

(define (trace-display . args)
  (display (trace-indent))
  (for-each (lambda (a)
	      (display a)
	      (display " "))
	    args)
  (newline))

(define (wrap-trace lam name)
  (lambda args
    (trace-display "--> " name)
    (set! trace-level (1+ trace-level))
    (catch #t
	   (lambda ()
	     (trace-display "args: " (object->string args))
	     (let ((res (apply lam args)))
	       (set! trace-level (1- trace-level))      
	       (trace-display "<-- " name)
	       (trace-display "result: " (object->string res))
	       res))
	   (lambda err
	     (set! trace-level (1- trace-level))
	     (apply throw err)))))

(define (wrap-trace-point lam . msg)
  (lambda args
    (apply trace-display msg)
    (trace-display "args: " (object->string args))
    (apply lam args)))

(define-macro (set-trace-level! name)
  `(set! ,name (wrap-trace ,name ,(symbol->string name))))

(define-macro (set-trace-point! name . opt)
  (let ((msg (if (null? opt)
		 (symbol->string name)
		 (car opt))))
    `(set! ,name (wrap-trace-point ,name ,msg))))
